# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.api_meta import APIMeta

__all__ = ["WebhookListResponse", "Data"]


class Data(BaseModel):
    id: str
    """Platform webhook ID"""

    created_at: datetime = FieldInfo(alias="createdAt")

    enabled: bool

    events: List[str]

    name: str

    url: str


class WebhookListResponse(BaseModel):
    data: List[Data]

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from .tenant_usage import TenantUsage
from ..shared.api_meta import APIMeta

__all__ = ["UsageRetrieveResponse"]


class UsageRetrieveResponse(BaseModel):
    """Usage statistics for a single tenant"""

    data: TenantUsage
    """Tenant usage statistics"""

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = ["UsageExportResponse", "UsageExportResponseItem"]


class UsageExportResponseItem(BaseModel):
    """Single row in usage export (JSON format)"""

    bounce_rate: float
    """Bounce rate (0-1)"""

    bounced: int
    """Emails that bounced"""

    delivered: int
    """Emails successfully delivered"""

    delivery_rate: float
    """Delivery rate (0-1)"""

    hard_failed: int
    """Emails that hard-failed"""

    held: int
    """Emails currently held"""

    sent: int
    """Total emails sent"""

    soft_failed: int
    """Emails that soft-failed"""

    status: Literal["active", "suspended", "archived"]
    """Current tenant status"""

    tenant_id: str
    """Unique tenant identifier"""

    tenant_name: str
    """Tenant display name"""

    external_id: Optional[str] = None
    """Your external ID for this tenant"""


UsageExportResponse: TypeAlias = List[UsageExportResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.api_meta import APIMeta

__all__ = ["CredentialUpdateResponse", "Data"]


class Data(BaseModel):
    id: int
    """Unique identifier for the credential"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """When the credential was created"""

    hold: bool
    """
    Whether the credential is on hold (disabled). When `true`, the credential cannot
    be used to send emails.
    """

    last_used_at: Optional[datetime] = FieldInfo(alias="lastUsedAt", default=None)
    """When the credential was last used to send an email"""

    name: str
    """Name of the credential"""

    type: Literal["smtp", "api"]
    """Type of credential:

    - `smtp` - For SMTP-based email sending
    - `api` - For API-based email sending
    """

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """When the credential was last updated"""

    key: Optional[str] = None
    """The credential key (secret). Only included when:

    - Creating a new credential (always returned)
    - Retrieving with `reveal=true`
    """

    smtp_username: Optional[str] = FieldInfo(alias="smtpUsername", default=None)
    """SMTP username for authentication.

    Only included for SMTP credentials when the key is revealed.
    """


class CredentialUpdateResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.api_meta import APIMeta

__all__ = ["WebhookRetrieveDeliveryResponse", "Data", "DataRequest", "DataResponse"]


class DataRequest(BaseModel):
    """Request details"""

    headers: Optional[Dict[str, str]] = None
    """Request headers including signature"""

    payload: Optional[Dict[str, object]] = None
    """The complete webhook payload that was sent"""


class DataResponse(BaseModel):
    """Response details"""

    body: Optional[str] = None
    """Response body (truncated if too large)"""

    duration: Optional[int] = None
    """Response time in milliseconds"""


class Data(BaseModel):
    id: str
    """Unique delivery ID"""

    attempt: int
    """Attempt number"""

    event: str
    """Event type"""

    request: DataRequest
    """Request details"""

    response: DataResponse
    """Response details"""

    status_code: Optional[int] = FieldInfo(alias="statusCode", default=None)
    """HTTP status code from your endpoint"""

    success: bool
    """Whether delivery was successful"""

    tenant_id: str = FieldInfo(alias="tenantId")
    """Tenant that triggered the event"""

    timestamp: datetime
    """When delivery was attempted"""

    url: str
    """Endpoint URL"""

    webhook_id: str = FieldInfo(alias="webhookId")
    """Platform webhook ID"""

    webhook_name: str = FieldInfo(alias="webhookName")
    """Platform webhook name"""

    will_retry: bool = FieldInfo(alias="willRetry")
    """Whether this will be retried"""


class WebhookRetrieveDeliveryResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from typing_extensions import Literal, TypedDict

__all__ = ["TenantUpdateParams"]


class TenantUpdateParams(TypedDict, total=False):
    metadata: Optional[Dict[str, Union[str, float, bool, None]]]
    """Custom key-value pairs. Useful for storing references to your internal systems.

    **Limits:**

    - Max 50 keys
    - Key names max 40 characters
    - String values max 500 characters
    - Total size max 8KB
    """

    name: str
    """Display name for the tenant"""

    status: Literal["active", "suspended", "archived"]
    """Tenant status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["WebhookListDeliveriesResponse"]


class WebhookListDeliveriesResponse(BaseModel):
    """Summary of a platform webhook delivery attempt"""

    id: str
    """Unique delivery ID"""

    attempt: int
    """Attempt number (1 for first attempt, higher for retries)"""

    event: Literal[
        "MessageSent",
        "MessageDelayed",
        "MessageDeliveryFailed",
        "MessageHeld",
        "MessageBounced",
        "MessageLinkClicked",
        "MessageLoaded",
        "DomainDNSError",
    ]
    """Event type"""

    status_code: Optional[int] = FieldInfo(alias="statusCode", default=None)
    """HTTP status code returned by your endpoint (null if connection failed)"""

    success: bool
    """Whether delivery was successful (2xx response)"""

    tenant_id: str = FieldInfo(alias="tenantId")
    """Tenant that triggered the event"""

    timestamp: datetime
    """When the delivery was attempted"""

    url: str
    """Endpoint URL the delivery was sent to"""

    webhook_id: str = FieldInfo(alias="webhookId")
    """Platform webhook ID"""

    will_retry: bool = FieldInfo(alias="willRetry")
    """Whether this delivery will be retried"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .tenant import Tenant
from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["TenantRetrieveResponse"]


class TenantRetrieveResponse(BaseModel):
    data: Tenant

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CredentialRetrieveParams"]


class CredentialRetrieveParams(TypedDict, total=False):
    tenant_id: Required[Annotated[str, PropertyInfo(alias="tenantId")]]

    reveal: bool
    """Set to `true` to include the credential key in the response"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import APIMeta as APIMeta
from .tenant import Tenant as Tenant
from .log_entry import LogEntry as LogEntry
from .email_rates import EmailRates as EmailRates
from .limits_data import LimitsData as LimitsData
from .email_counts import EmailCounts as EmailCounts
from .usage_period import UsagePeriod as UsagePeriod
from .log_list_params import LogListParams as LogListParams
from .log_entry_detail import LogEntryDetail as LogEntryDetail
from .email_list_params import EmailListParams as EmailListParams
from .email_send_params import EmailSendParams as EmailSendParams
from .org_usage_summary import OrgUsageSummary as OrgUsageSummary
from .tenant_usage_item import TenantUsageItem as TenantUsageItem
from .tenant_list_params import TenantListParams as TenantListParams
from .email_list_response import EmailListResponse as EmailListResponse
from .email_send_response import EmailSendResponse as EmailSendResponse
from .usage_export_params import UsageExportParams as UsageExportParams
from .email_retry_response import EmailRetryResponse as EmailRetryResponse
from .tenant_create_params import TenantCreateParams as TenantCreateParams
from .tenant_update_params import TenantUpdateParams as TenantUpdateParams
from .email_retrieve_params import EmailRetrieveParams as EmailRetrieveParams
from .email_send_raw_params import EmailSendRawParams as EmailSendRawParams
from .log_retrieve_response import LogRetrieveResponse as LogRetrieveResponse
from .usage_export_response import UsageExportResponse as UsageExportResponse
from .usage_retrieve_params import UsageRetrieveParams as UsageRetrieveParams
from .tenant_create_response import TenantCreateResponse as TenantCreateResponse
from .tenant_delete_response import TenantDeleteResponse as TenantDeleteResponse
from .tenant_update_response import TenantUpdateResponse as TenantUpdateResponse
from .email_retrieve_response import EmailRetrieveResponse as EmailRetrieveResponse
from .email_send_batch_params import EmailSendBatchParams as EmailSendBatchParams
from .email_send_raw_response import EmailSendRawResponse as EmailSendRawResponse
from .limit_retrieve_response import LimitRetrieveResponse as LimitRetrieveResponse
from .tenant_retrieve_response import TenantRetrieveResponse as TenantRetrieveResponse
from .email_send_batch_response import EmailSendBatchResponse as EmailSendBatchResponse
from .usage_list_tenants_params import UsageListTenantsParams as UsageListTenantsParams
from .email_retrieve_deliveries_response import EmailRetrieveDeliveriesResponse as EmailRetrieveDeliveriesResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .dns_record import DNSRecord as DNSRecord
from .tenant_usage import TenantUsage as TenantUsage
from .track_domain import TrackDomain as TrackDomain
from .webhook_test_params import WebhookTestParams as WebhookTestParams
from .domain_create_params import DomainCreateParams as DomainCreateParams
from .domain_list_response import DomainListResponse as DomainListResponse
from .usage_retrieve_params import UsageRetrieveParams as UsageRetrieveParams
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_list_response import WebhookListResponse as WebhookListResponse
from .webhook_test_response import WebhookTestResponse as WebhookTestResponse
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .credential_list_params import CredentialListParams as CredentialListParams
from .domain_create_response import DomainCreateResponse as DomainCreateResponse
from .domain_delete_response import DomainDeleteResponse as DomainDeleteResponse
from .domain_verify_response import DomainVerifyResponse as DomainVerifyResponse
from .tracking_create_params import TrackingCreateParams as TrackingCreateParams
from .tracking_list_response import TrackingListResponse as TrackingListResponse
from .tracking_update_params import TrackingUpdateParams as TrackingUpdateParams
from .suppression_list_params import SuppressionListParams as SuppressionListParams
from .tenant_usage_timeseries import TenantUsageTimeseries as TenantUsageTimeseries
from .usage_retrieve_response import UsageRetrieveResponse as UsageRetrieveResponse
from .webhook_create_response import WebhookCreateResponse as WebhookCreateResponse
from .webhook_delete_response import WebhookDeleteResponse as WebhookDeleteResponse
from .webhook_update_response import WebhookUpdateResponse as WebhookUpdateResponse
from .credential_create_params import CredentialCreateParams as CredentialCreateParams
from .credential_list_response import CredentialListResponse as CredentialListResponse
from .credential_update_params import CredentialUpdateParams as CredentialUpdateParams
from .domain_retrieve_response import DomainRetrieveResponse as DomainRetrieveResponse
from .tracking_create_response import TrackingCreateResponse as TrackingCreateResponse
from .tracking_delete_response import TrackingDeleteResponse as TrackingDeleteResponse
from .tracking_update_response import TrackingUpdateResponse as TrackingUpdateResponse
from .tracking_verify_response import TrackingVerifyResponse as TrackingVerifyResponse
from .suppression_create_params import SuppressionCreateParams as SuppressionCreateParams
from .suppression_list_response import SuppressionListResponse as SuppressionListResponse
from .webhook_retrieve_response import WebhookRetrieveResponse as WebhookRetrieveResponse
from .credential_create_response import CredentialCreateResponse as CredentialCreateResponse
from .credential_delete_response import CredentialDeleteResponse as CredentialDeleteResponse
from .credential_retrieve_params import CredentialRetrieveParams as CredentialRetrieveParams
from .credential_update_response import CredentialUpdateResponse as CredentialUpdateResponse
from .tracking_retrieve_response import TrackingRetrieveResponse as TrackingRetrieveResponse
from .suppression_create_response import SuppressionCreateResponse as SuppressionCreateResponse
from .suppression_delete_response import SuppressionDeleteResponse as SuppressionDeleteResponse
from .credential_retrieve_response import CredentialRetrieveResponse as CredentialRetrieveResponse
from .suppression_retrieve_response import SuppressionRetrieveResponse as SuppressionRetrieveResponse
from .webhook_list_deliveries_params import WebhookListDeliveriesParams as WebhookListDeliveriesParams
from .usage_retrieve_timeseries_params import UsageRetrieveTimeseriesParams as UsageRetrieveTimeseriesParams
from .webhook_list_deliveries_response import WebhookListDeliveriesResponse as WebhookListDeliveriesResponse
from .webhook_replay_delivery_response import WebhookReplayDeliveryResponse as WebhookReplayDeliveryResponse
from .usage_retrieve_timeseries_response import UsageRetrieveTimeseriesResponse as UsageRetrieveTimeseriesResponse
from .webhook_retrieve_delivery_response import WebhookRetrieveDeliveryResponse as WebhookRetrieveDeliveryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .email_rates import EmailRates
from .email_counts import EmailCounts
from .usage_period import UsagePeriod
from .shared.api_meta import APIMeta

__all__ = ["OrgUsageSummary", "Data", "DataTenants"]


class DataTenants(BaseModel):
    active: int
    """Number of active tenants"""

    total: int
    """Total number of tenants"""

    with_activity: int = FieldInfo(alias="withActivity")
    """Number of tenants with sending activity"""


class Data(BaseModel):
    emails: EmailCounts
    """Email delivery counts"""

    period: UsagePeriod
    """Time period for usage data"""

    rates: EmailRates
    """Email delivery rates (as decimals, e.g., 0.95 = 95%)"""

    tenants: DataTenants


class OrgUsageSummary(BaseModel):
    """Org-wide usage summary response"""

    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from typing_extensions import Literal

import httpx

from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from ...types import tenant_list_params, tenant_create_params, tenant_update_params
from .domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .tracking import (
    TrackingResource,
    AsyncTrackingResource,
    TrackingResourceWithRawResponse,
    AsyncTrackingResourceWithRawResponse,
    TrackingResourceWithStreamingResponse,
    AsyncTrackingResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .credentials import (
    CredentialsResource,
    AsyncCredentialsResource,
    CredentialsResourceWithRawResponse,
    AsyncCredentialsResourceWithRawResponse,
    CredentialsResourceWithStreamingResponse,
    AsyncCredentialsResourceWithStreamingResponse,
)
from ...pagination import SyncPageNumberPagination, AsyncPageNumberPagination
from .suppressions import (
    SuppressionsResource,
    AsyncSuppressionsResource,
    SuppressionsResourceWithRawResponse,
    AsyncSuppressionsResourceWithRawResponse,
    SuppressionsResourceWithStreamingResponse,
    AsyncSuppressionsResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.tenant import Tenant
from ...types.tenant_create_response import TenantCreateResponse
from ...types.tenant_delete_response import TenantDeleteResponse
from ...types.tenant_update_response import TenantUpdateResponse
from ...types.tenant_retrieve_response import TenantRetrieveResponse

__all__ = ["TenantsResource", "AsyncTenantsResource"]


class TenantsResource(SyncAPIResource):
    @cached_property
    def credentials(self) -> CredentialsResource:
        return CredentialsResource(self._client)

    @cached_property
    def domains(self) -> DomainsResource:
        return DomainsResource(self._client)

    @cached_property
    def suppressions(self) -> SuppressionsResource:
        return SuppressionsResource(self._client)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        return WebhooksResource(self._client)

    @cached_property
    def tracking(self) -> TrackingResource:
        return TrackingResource(self._client)

    @cached_property
    def usage(self) -> UsageResource:
        return UsageResource(self._client)

    @cached_property
    def with_raw_response(self) -> TenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return TenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return TenantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        metadata: Optional[Dict[str, Union[str, float, bool, None]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantCreateResponse:
        """Create a new tenant.

        Returns the created tenant with a unique `id`.

        Store this ID in your database to
        reference this tenant later.

        Args:
          name: Display name for the tenant (e.g., your customer's company name)

          metadata: Custom key-value pairs. Useful for storing references to your internal systems.

              **Limits:**

              - Max 50 keys
              - Key names max 40 characters
              - String values max 500 characters
              - Total size max 8KB

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/tenants",
            body=maybe_transform(
                {
                    "name": name,
                    "metadata": metadata,
                },
                tenant_create_params.TenantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantCreateResponse,
        )

    def retrieve(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantRetrieveResponse:
        """
        Get a tenant by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get(
            f"/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantRetrieveResponse,
        )

    def update(
        self,
        tenant_id: str,
        *,
        metadata: Optional[Dict[str, Union[str, float, bool, None]]] | Omit = omit,
        name: str | Omit = omit,
        status: Literal["active", "suspended", "archived"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantUpdateResponse:
        """Update a tenant's name, metadata, or status.

        At least one field is required.

        Metadata is replaced entirely—include all keys you want to keep.

        Args:
          metadata: Custom key-value pairs. Useful for storing references to your internal systems.

              **Limits:**

              - Max 50 keys
              - Key names max 40 characters
              - String values max 500 characters
              - Total size max 8KB

          name: Display name for the tenant

          status: Tenant status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._patch(
            f"/tenants/{tenant_id}",
            body=maybe_transform(
                {
                    "metadata": metadata,
                    "name": name,
                    "status": status,
                },
                tenant_update_params.TenantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantUpdateResponse,
        )

    def list(
        self,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["active", "suspended", "archived"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPageNumberPagination[Tenant]:
        """List all tenants with pagination.

        Filter by `status` if needed.

        Args:
          page: Page number (1-indexed)

          per_page: Number of items per page (max 100)

          status: Filter by tenant status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/tenants",
            page=SyncPageNumberPagination[Tenant],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    tenant_list_params.TenantListParams,
                ),
            ),
            model=Tenant,
        )

    def delete(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantDeleteResponse:
        """Permanently delete a tenant.

        This cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._delete(
            f"/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantDeleteResponse,
        )


class AsyncTenantsResource(AsyncAPIResource):
    @cached_property
    def credentials(self) -> AsyncCredentialsResource:
        return AsyncCredentialsResource(self._client)

    @cached_property
    def domains(self) -> AsyncDomainsResource:
        return AsyncDomainsResource(self._client)

    @cached_property
    def suppressions(self) -> AsyncSuppressionsResource:
        return AsyncSuppressionsResource(self._client)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        return AsyncWebhooksResource(self._client)

    @cached_property
    def tracking(self) -> AsyncTrackingResource:
        return AsyncTrackingResource(self._client)

    @cached_property
    def usage(self) -> AsyncUsageResource:
        return AsyncUsageResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return AsyncTenantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        metadata: Optional[Dict[str, Union[str, float, bool, None]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantCreateResponse:
        """Create a new tenant.

        Returns the created tenant with a unique `id`.

        Store this ID in your database to
        reference this tenant later.

        Args:
          name: Display name for the tenant (e.g., your customer's company name)

          metadata: Custom key-value pairs. Useful for storing references to your internal systems.

              **Limits:**

              - Max 50 keys
              - Key names max 40 characters
              - String values max 500 characters
              - Total size max 8KB

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/tenants",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "metadata": metadata,
                },
                tenant_create_params.TenantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantCreateResponse,
        )

    async def retrieve(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantRetrieveResponse:
        """
        Get a tenant by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._get(
            f"/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantRetrieveResponse,
        )

    async def update(
        self,
        tenant_id: str,
        *,
        metadata: Optional[Dict[str, Union[str, float, bool, None]]] | Omit = omit,
        name: str | Omit = omit,
        status: Literal["active", "suspended", "archived"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantUpdateResponse:
        """Update a tenant's name, metadata, or status.

        At least one field is required.

        Metadata is replaced entirely—include all keys you want to keep.

        Args:
          metadata: Custom key-value pairs. Useful for storing references to your internal systems.

              **Limits:**

              - Max 50 keys
              - Key names max 40 characters
              - String values max 500 characters
              - Total size max 8KB

          name: Display name for the tenant

          status: Tenant status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._patch(
            f"/tenants/{tenant_id}",
            body=await async_maybe_transform(
                {
                    "metadata": metadata,
                    "name": name,
                    "status": status,
                },
                tenant_update_params.TenantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantUpdateResponse,
        )

    def list(
        self,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["active", "suspended", "archived"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Tenant, AsyncPageNumberPagination[Tenant]]:
        """List all tenants with pagination.

        Filter by `status` if needed.

        Args:
          page: Page number (1-indexed)

          per_page: Number of items per page (max 100)

          status: Filter by tenant status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/tenants",
            page=AsyncPageNumberPagination[Tenant],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    tenant_list_params.TenantListParams,
                ),
            ),
            model=Tenant,
        )

    async def delete(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantDeleteResponse:
        """Permanently delete a tenant.

        This cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._delete(
            f"/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantDeleteResponse,
        )


class TenantsResourceWithRawResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_raw_response_wrapper(
            tenants.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tenants.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tenants.update,
        )
        self.list = to_raw_response_wrapper(
            tenants.list,
        )
        self.delete = to_raw_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def credentials(self) -> CredentialsResourceWithRawResponse:
        return CredentialsResourceWithRawResponse(self._tenants.credentials)

    @cached_property
    def domains(self) -> DomainsResourceWithRawResponse:
        return DomainsResourceWithRawResponse(self._tenants.domains)

    @cached_property
    def suppressions(self) -> SuppressionsResourceWithRawResponse:
        return SuppressionsResourceWithRawResponse(self._tenants.suppressions)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithRawResponse:
        return WebhooksResourceWithRawResponse(self._tenants.webhooks)

    @cached_property
    def tracking(self) -> TrackingResourceWithRawResponse:
        return TrackingResourceWithRawResponse(self._tenants.tracking)

    @cached_property
    def usage(self) -> UsageResourceWithRawResponse:
        return UsageResourceWithRawResponse(self._tenants.usage)


class AsyncTenantsResourceWithRawResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_raw_response_wrapper(
            tenants.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tenants.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tenants.update,
        )
        self.list = async_to_raw_response_wrapper(
            tenants.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def credentials(self) -> AsyncCredentialsResourceWithRawResponse:
        return AsyncCredentialsResourceWithRawResponse(self._tenants.credentials)

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithRawResponse:
        return AsyncDomainsResourceWithRawResponse(self._tenants.domains)

    @cached_property
    def suppressions(self) -> AsyncSuppressionsResourceWithRawResponse:
        return AsyncSuppressionsResourceWithRawResponse(self._tenants.suppressions)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithRawResponse:
        return AsyncWebhooksResourceWithRawResponse(self._tenants.webhooks)

    @cached_property
    def tracking(self) -> AsyncTrackingResourceWithRawResponse:
        return AsyncTrackingResourceWithRawResponse(self._tenants.tracking)

    @cached_property
    def usage(self) -> AsyncUsageResourceWithRawResponse:
        return AsyncUsageResourceWithRawResponse(self._tenants.usage)


class TenantsResourceWithStreamingResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_streamed_response_wrapper(
            tenants.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tenants.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tenants.update,
        )
        self.list = to_streamed_response_wrapper(
            tenants.list,
        )
        self.delete = to_streamed_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def credentials(self) -> CredentialsResourceWithStreamingResponse:
        return CredentialsResourceWithStreamingResponse(self._tenants.credentials)

    @cached_property
    def domains(self) -> DomainsResourceWithStreamingResponse:
        return DomainsResourceWithStreamingResponse(self._tenants.domains)

    @cached_property
    def suppressions(self) -> SuppressionsResourceWithStreamingResponse:
        return SuppressionsResourceWithStreamingResponse(self._tenants.suppressions)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithStreamingResponse:
        return WebhooksResourceWithStreamingResponse(self._tenants.webhooks)

    @cached_property
    def tracking(self) -> TrackingResourceWithStreamingResponse:
        return TrackingResourceWithStreamingResponse(self._tenants.tracking)

    @cached_property
    def usage(self) -> UsageResourceWithStreamingResponse:
        return UsageResourceWithStreamingResponse(self._tenants.usage)


class AsyncTenantsResourceWithStreamingResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_streamed_response_wrapper(
            tenants.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tenants.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tenants.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tenants.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tenants.delete,
        )

    @cached_property
    def credentials(self) -> AsyncCredentialsResourceWithStreamingResponse:
        return AsyncCredentialsResourceWithStreamingResponse(self._tenants.credentials)

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithStreamingResponse:
        return AsyncDomainsResourceWithStreamingResponse(self._tenants.domains)

    @cached_property
    def suppressions(self) -> AsyncSuppressionsResourceWithStreamingResponse:
        return AsyncSuppressionsResourceWithStreamingResponse(self._tenants.suppressions)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithStreamingResponse:
        return AsyncWebhooksResourceWithStreamingResponse(self._tenants.webhooks)

    @cached_property
    def tracking(self) -> AsyncTrackingResourceWithStreamingResponse:
        return AsyncTrackingResourceWithStreamingResponse(self._tenants.tracking)

    @cached_property
    def usage(self) -> AsyncUsageResourceWithStreamingResponse:
        return AsyncUsageResourceWithStreamingResponse(self._tenants.usage)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CredentialListParams"]


class CredentialListParams(TypedDict, total=False):
    page: int
    """Page number (1-indexed)"""

    per_page: Annotated[int, PropertyInfo(alias="perPage")]
    """Number of items per page (max 100)"""

    type: Literal["smtp", "api"]
    """Filter by credential type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import ArkError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import logs, usage, emails, limits, tenants, platform
    from .resources.logs import LogsResource, AsyncLogsResource
    from .resources.usage import UsageResource, AsyncUsageResource
    from .resources.emails import EmailsResource, AsyncEmailsResource
    from .resources.limits import LimitsResource, AsyncLimitsResource
    from .resources.tenants.tenants import TenantsResource, AsyncTenantsResource
    from .resources.platform.platform import PlatformResource, AsyncPlatformResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Ark", "AsyncArk", "Client", "AsyncClient"]


class Ark(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Ark client instance.

        This automatically infers the `api_key` argument from the `ARK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ARK_API_KEY")
        if api_key is None:
            raise ArkError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ARK_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ARK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.arkhq.io/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def emails(self) -> EmailsResource:
        from .resources.emails import EmailsResource

        return EmailsResource(self)

    @cached_property
    def logs(self) -> LogsResource:
        from .resources.logs import LogsResource

        return LogsResource(self)

    @cached_property
    def usage(self) -> UsageResource:
        from .resources.usage import UsageResource

        return UsageResource(self)

    @cached_property
    def limits(self) -> LimitsResource:
        from .resources.limits import LimitsResource

        return LimitsResource(self)

    @cached_property
    def tenants(self) -> TenantsResource:
        from .resources.tenants import TenantsResource

        return TenantsResource(self)

    @cached_property
    def platform(self) -> PlatformResource:
        from .resources.platform import PlatformResource

        return PlatformResource(self)

    @cached_property
    def with_raw_response(self) -> ArkWithRawResponse:
        return ArkWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ArkWithStreamedResponse:
        return ArkWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncArk(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncArk client instance.

        This automatically infers the `api_key` argument from the `ARK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ARK_API_KEY")
        if api_key is None:
            raise ArkError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ARK_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ARK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.arkhq.io/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def emails(self) -> AsyncEmailsResource:
        from .resources.emails import AsyncEmailsResource

        return AsyncEmailsResource(self)

    @cached_property
    def logs(self) -> AsyncLogsResource:
        from .resources.logs import AsyncLogsResource

        return AsyncLogsResource(self)

    @cached_property
    def usage(self) -> AsyncUsageResource:
        from .resources.usage import AsyncUsageResource

        return AsyncUsageResource(self)

    @cached_property
    def limits(self) -> AsyncLimitsResource:
        from .resources.limits import AsyncLimitsResource

        return AsyncLimitsResource(self)

    @cached_property
    def tenants(self) -> AsyncTenantsResource:
        from .resources.tenants import AsyncTenantsResource

        return AsyncTenantsResource(self)

    @cached_property
    def platform(self) -> AsyncPlatformResource:
        from .resources.platform import AsyncPlatformResource

        return AsyncPlatformResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncArkWithRawResponse:
        return AsyncArkWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncArkWithStreamedResponse:
        return AsyncArkWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ArkWithRawResponse:
    _client: Ark

    def __init__(self, client: Ark) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.EmailsResourceWithRawResponse:
        from .resources.emails import EmailsResourceWithRawResponse

        return EmailsResourceWithRawResponse(self._client.emails)

    @cached_property
    def logs(self) -> logs.LogsResourceWithRawResponse:
        from .resources.logs import LogsResourceWithRawResponse

        return LogsResourceWithRawResponse(self._client.logs)

    @cached_property
    def usage(self) -> usage.UsageResourceWithRawResponse:
        from .resources.usage import UsageResourceWithRawResponse

        return UsageResourceWithRawResponse(self._client.usage)

    @cached_property
    def limits(self) -> limits.LimitsResourceWithRawResponse:
        from .resources.limits import LimitsResourceWithRawResponse

        return LimitsResourceWithRawResponse(self._client.limits)

    @cached_property
    def tenants(self) -> tenants.TenantsResourceWithRawResponse:
        from .resources.tenants import TenantsResourceWithRawResponse

        return TenantsResourceWithRawResponse(self._client.tenants)

    @cached_property
    def platform(self) -> platform.PlatformResourceWithRawResponse:
        from .resources.platform import PlatformResourceWithRawResponse

        return PlatformResourceWithRawResponse(self._client.platform)


class AsyncArkWithRawResponse:
    _client: AsyncArk

    def __init__(self, client: AsyncArk) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.AsyncEmailsResourceWithRawResponse:
        from .resources.emails import AsyncEmailsResourceWithRawResponse

        return AsyncEmailsResourceWithRawResponse(self._client.emails)

    @cached_property
    def logs(self) -> logs.AsyncLogsResourceWithRawResponse:
        from .resources.logs import AsyncLogsResourceWithRawResponse

        return AsyncLogsResourceWithRawResponse(self._client.logs)

    @cached_property
    def usage(self) -> usage.AsyncUsageResourceWithRawResponse:
        from .resources.usage import AsyncUsageResourceWithRawResponse

        return AsyncUsageResourceWithRawResponse(self._client.usage)

    @cached_property
    def limits(self) -> limits.AsyncLimitsResourceWithRawResponse:
        from .resources.limits import AsyncLimitsResourceWithRawResponse

        return AsyncLimitsResourceWithRawResponse(self._client.limits)

    @cached_property
    def tenants(self) -> tenants.AsyncTenantsResourceWithRawResponse:
        from .resources.tenants import AsyncTenantsResourceWithRawResponse

        return AsyncTenantsResourceWithRawResponse(self._client.tenants)

    @cached_property
    def platform(self) -> platform.AsyncPlatformResourceWithRawResponse:
        from .resources.platform import AsyncPlatformResourceWithRawResponse

        return AsyncPlatformResourceWithRawResponse(self._client.platform)


class ArkWithStreamedResponse:
    _client: Ark

    def __init__(self, client: Ark) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.EmailsResourceWithStreamingResponse:
        from .resources.emails import EmailsResourceWithStreamingResponse

        return EmailsResourceWithStreamingResponse(self._client.emails)

    @cached_property
    def logs(self) -> logs.LogsResourceWithStreamingResponse:
        from .resources.logs import LogsResourceWithStreamingResponse

        return LogsResourceWithStreamingResponse(self._client.logs)

    @cached_property
    def usage(self) -> usage.UsageResourceWithStreamingResponse:
        from .resources.usage import UsageResourceWithStreamingResponse

        return UsageResourceWithStreamingResponse(self._client.usage)

    @cached_property
    def limits(self) -> limits.LimitsResourceWithStreamingResponse:
        from .resources.limits import LimitsResourceWithStreamingResponse

        return LimitsResourceWithStreamingResponse(self._client.limits)

    @cached_property
    def tenants(self) -> tenants.TenantsResourceWithStreamingResponse:
        from .resources.tenants import TenantsResourceWithStreamingResponse

        return TenantsResourceWithStreamingResponse(self._client.tenants)

    @cached_property
    def platform(self) -> platform.PlatformResourceWithStreamingResponse:
        from .resources.platform import PlatformResourceWithStreamingResponse

        return PlatformResourceWithStreamingResponse(self._client.platform)


class AsyncArkWithStreamedResponse:
    _client: AsyncArk

    def __init__(self, client: AsyncArk) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.AsyncEmailsResourceWithStreamingResponse:
        from .resources.emails import AsyncEmailsResourceWithStreamingResponse

        return AsyncEmailsResourceWithStreamingResponse(self._client.emails)

    @cached_property
    def logs(self) -> logs.AsyncLogsResourceWithStreamingResponse:
        from .resources.logs import AsyncLogsResourceWithStreamingResponse

        return AsyncLogsResourceWithStreamingResponse(self._client.logs)

    @cached_property
    def usage(self) -> usage.AsyncUsageResourceWithStreamingResponse:
        from .resources.usage import AsyncUsageResourceWithStreamingResponse

        return AsyncUsageResourceWithStreamingResponse(self._client.usage)

    @cached_property
    def limits(self) -> limits.AsyncLimitsResourceWithStreamingResponse:
        from .resources.limits import AsyncLimitsResourceWithStreamingResponse

        return AsyncLimitsResourceWithStreamingResponse(self._client.limits)

    @cached_property
    def tenants(self) -> tenants.AsyncTenantsResourceWithStreamingResponse:
        from .resources.tenants import AsyncTenantsResourceWithStreamingResponse

        return AsyncTenantsResourceWithStreamingResponse(self._client.tenants)

    @cached_property
    def platform(self) -> platform.AsyncPlatformResourceWithStreamingResponse:
        from .resources.platform import AsyncPlatformResourceWithStreamingResponse

        return AsyncPlatformResourceWithStreamingResponse(self._client.platform)


Client = Ark

AsyncClient = AsyncArk



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from typing_extensions import Required, TypedDict

__all__ = ["TenantCreateParams"]


class TenantCreateParams(TypedDict, total=False):
    name: Required[str]
    """Display name for the tenant (e.g., your customer's company name)"""

    metadata: Optional[Dict[str, Union[str, float, bool, None]]]
    """Custom key-value pairs. Useful for storing references to your internal systems.

    **Limits:**

    - Max 50 keys
    - Key names max 40 characters
    - String values max 500 characters
    - Total size max 8KB
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["EmailCounts"]


class EmailCounts(BaseModel):
    """Email delivery counts"""

    bounced: int
    """Emails that bounced"""

    delivered: int
    """Emails successfully delivered"""

    hard_failed: int
    """Emails that hard-failed (permanent failures)"""

    held: int
    """Emails currently held for review"""

    sent: int
    """Total emails sent"""

    soft_failed: int
    """Emails that soft-failed (temporary failures, may be retried)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UsageExportParams"]


class UsageExportParams(TypedDict, total=False):
    format: Literal["csv", "jsonl"]
    """Export format"""

    min_sent: Annotated[int, PropertyInfo(alias="minSent")]
    """Only include tenants with at least this many emails sent"""

    period: str
    """Time period for export.

    **Shortcuts:** `this_month`, `last_month`, `last_30_days`, etc.

    **Month format:** `2024-01` (YYYY-MM)

    **Custom range:** `2024-01-01..2024-01-15`
    """

    status: Literal["active", "suspended", "archived"]
    """Filter by tenant status"""

    timezone: str
    """Timezone for period calculations (IANA format)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["EmailRates"]


class EmailRates(BaseModel):
    """Email delivery rates (as decimals, e.g., 0.95 = 95%)"""

    bounce_rate: float
    """Percentage of sent emails that bounced (0-1)"""

    delivery_rate: float
    """Percentage of sent emails that were delivered (0-1)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["UsageRetrieveTimeseriesParams"]


class UsageRetrieveTimeseriesParams(TypedDict, total=False):
    granularity: Literal["hour", "day", "week", "month"]
    """Time bucket size for data points"""

    period: str
    """Time period for timeseries data. Defaults to current month."""

    timezone: str
    """Timezone for period calculations (IANA format). Defaults to UTC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UsageListTenantsParams"]


class UsageListTenantsParams(TypedDict, total=False):
    min_sent: Annotated[int, PropertyInfo(alias="minSent")]
    """Only include tenants with at least this many emails sent"""

    page: int
    """Page number (1-indexed)"""

    period: str
    """Time period for usage data. Defaults to current month.

    **Shortcuts:** `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
    `last_month`, `last_7_days`, `last_30_days`, `last_90_days`

    **Month format:** `2024-01` (YYYY-MM)

    **Custom range:** `2024-01-01..2024-01-15`
    """

    per_page: Annotated[int, PropertyInfo(alias="perPage")]
    """Results per page (max 100)"""

    sort: Literal[
        "sent",
        "-sent",
        "delivered",
        "-delivered",
        "bounce_rate",
        "-bounce_rate",
        "delivery_rate",
        "-delivery_rate",
        "tenant_name",
        "-tenant_name",
    ]
    """Sort order for results. Prefix with `-` for descending order."""

    status: Literal["active", "suspended", "archived"]
    """Filter by tenant status"""

    timezone: str
    """Timezone for period calculations (IANA format). Defaults to UTC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from ark import Ark, AsyncArk
from ark.types import (
    OrgUsageSummary,
    TenantUsageItem,
    UsageExportResponse,
)
from tests.utils import assert_matches_type
from ark.pagination import SyncPageNumberPagination, AsyncPageNumberPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsage:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Ark) -> None:
        usage = client.usage.retrieve()
        assert_matches_type(OrgUsageSummary, usage, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: Ark) -> None:
        usage = client.usage.retrieve(
            period="period",
            timezone="timezone",
        )
        assert_matches_type(OrgUsageSummary, usage, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Ark) -> None:
        response = client.usage.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(OrgUsageSummary, usage, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Ark) -> None:
        with client.usage.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(OrgUsageSummary, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_export(self, client: Ark) -> None:
        usage = client.usage.export()
        assert_matches_type(UsageExportResponse, usage, path=["response"])

    @parametrize
    def test_method_export_with_all_params(self, client: Ark) -> None:
        usage = client.usage.export(
            format="csv",
            min_sent=0,
            period="period",
            status="active",
            timezone="timezone",
        )
        assert_matches_type(UsageExportResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_export(self, client: Ark) -> None:
        response = client.usage.with_raw_response.export()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageExportResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_export(self, client: Ark) -> None:
        with client.usage.with_streaming_response.export() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageExportResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list_tenants(self, client: Ark) -> None:
        usage = client.usage.list_tenants()
        assert_matches_type(SyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

    @parametrize
    def test_method_list_tenants_with_all_params(self, client: Ark) -> None:
        usage = client.usage.list_tenants(
            min_sent=0,
            page=1,
            period="period",
            per_page=1,
            sort="sent",
            status="active",
            timezone="timezone",
        )
        assert_matches_type(SyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

    @parametrize
    def test_raw_response_list_tenants(self, client: Ark) -> None:
        response = client.usage.with_raw_response.list_tenants()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(SyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

    @parametrize
    def test_streaming_response_list_tenants(self, client: Ark) -> None:
        with client.usage.with_streaming_response.list_tenants() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(SyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUsage:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncArk) -> None:
        usage = await async_client.usage.retrieve()
        assert_matches_type(OrgUsageSummary, usage, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncArk) -> None:
        usage = await async_client.usage.retrieve(
            period="period",
            timezone="timezone",
        )
        assert_matches_type(OrgUsageSummary, usage, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncArk) -> None:
        response = await async_client.usage.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(OrgUsageSummary, usage, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncArk) -> None:
        async with async_client.usage.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(OrgUsageSummary, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_export(self, async_client: AsyncArk) -> None:
        usage = await async_client.usage.export()
        assert_matches_type(UsageExportResponse, usage, path=["response"])

    @parametrize
    async def test_method_export_with_all_params(self, async_client: AsyncArk) -> None:
        usage = await async_client.usage.export(
            format="csv",
            min_sent=0,
            period="period",
            status="active",
            timezone="timezone",
        )
        assert_matches_type(UsageExportResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_export(self, async_client: AsyncArk) -> None:
        response = await async_client.usage.with_raw_response.export()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageExportResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_export(self, async_client: AsyncArk) -> None:
        async with async_client.usage.with_streaming_response.export() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageExportResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list_tenants(self, async_client: AsyncArk) -> None:
        usage = await async_client.usage.list_tenants()
        assert_matches_type(AsyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

    @parametrize
    async def test_method_list_tenants_with_all_params(self, async_client: AsyncArk) -> None:
        usage = await async_client.usage.list_tenants(
            min_sent=0,
            page=1,
            period="period",
            per_page=1,
            sort="sent",
            status="active",
            timezone="timezone",
        )
        assert_matches_type(AsyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

    @parametrize
    async def test_raw_response_list_tenants(self, async_client: AsyncArk) -> None:
        response = await async_client.usage.with_raw_response.list_tenants()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(AsyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

    @parametrize
    async def test_streaming_response_list_tenants(self, async_client: AsyncArk) -> None:
        async with async_client.usage.with_streaming_response.list_tenants() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(AsyncPageNumberPagination[TenantUsageItem], usage, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .platform import (
    PlatformResource,
    AsyncPlatformResource,
    PlatformResourceWithRawResponse,
    AsyncPlatformResourceWithRawResponse,
    PlatformResourceWithStreamingResponse,
    AsyncPlatformResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)

__all__ = [
    "WebhooksResource",
    "AsyncWebhooksResource",
    "WebhooksResourceWithRawResponse",
    "AsyncWebhooksResourceWithRawResponse",
    "WebhooksResourceWithStreamingResponse",
    "AsyncWebhooksResourceWithStreamingResponse",
    "PlatformResource",
    "AsyncPlatformResource",
    "PlatformResourceWithRawResponse",
    "AsyncPlatformResourceWithRawResponse",
    "PlatformResourceWithStreamingResponse",
    "AsyncPlatformResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .webhook_test_params import WebhookTestParams as WebhookTestParams
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_list_response import WebhookListResponse as WebhookListResponse
from .webhook_test_response import WebhookTestResponse as WebhookTestResponse
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .webhook_create_response import WebhookCreateResponse as WebhookCreateResponse
from .webhook_delete_response import WebhookDeleteResponse as WebhookDeleteResponse
from .webhook_update_response import WebhookUpdateResponse as WebhookUpdateResponse
from .webhook_retrieve_response import WebhookRetrieveResponse as WebhookRetrieveResponse
from .webhook_list_deliveries_params import WebhookListDeliveriesParams as WebhookListDeliveriesParams
from .webhook_list_deliveries_response import WebhookListDeliveriesResponse as WebhookListDeliveriesResponse
from .webhook_replay_delivery_response import WebhookReplayDeliveryResponse as WebhookReplayDeliveryResponse
from .webhook_retrieve_delivery_response import WebhookRetrieveDeliveryResponse as WebhookRetrieveDeliveryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Tenant"]


class Tenant(BaseModel):
    id: str
    """Unique identifier for the tenant"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """When the tenant was created"""

    metadata: Dict[str, Union[str, float, bool, None]]
    """Custom key-value pairs for storing additional data"""

    name: str
    """Display name for the tenant"""

    status: Literal["active", "suspended", "archived"]
    """Current status of the tenant:

    - `active` - Normal operation
    - `suspended` - Temporarily disabled
    - `archived` - Soft-deleted
    """

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """When the tenant was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.api_meta import APIMeta

__all__ = ["SuppressionDeleteResponse", "Data"]


class Data(BaseModel):
    message: str


class SuppressionDeleteResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["UsageRetrieveParams"]


class UsageRetrieveParams(TypedDict, total=False):
    period: str
    """Time period for usage data.

    **Shortcuts:** `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
    `last_month`, `last_7_days`, `last_30_days`, `last_90_days`

    **Month format:** `2024-01` (YYYY-MM)

    **Custom range:** `2024-01-01..2024-01-15`
    """

    timezone: str
    """Timezone for period calculations (IANA format)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TenantListParams"]


class TenantListParams(TypedDict, total=False):
    page: int
    """Page number (1-indexed)"""

    per_page: Annotated[int, PropertyInfo(alias="perPage")]
    """Number of items per page (max 100)"""

    status: Literal["active", "suspended", "archived"]
    """Filter by tenant status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["PlatformResource", "AsyncPlatformResource"]


class PlatformResource(SyncAPIResource):
    @cached_property
    def webhooks(self) -> WebhooksResource:
        return WebhooksResource(self._client)

    @cached_property
    def with_raw_response(self) -> PlatformResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return PlatformResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlatformResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return PlatformResourceWithStreamingResponse(self)


class AsyncPlatformResource(AsyncAPIResource):
    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        return AsyncWebhooksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPlatformResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlatformResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlatformResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return AsyncPlatformResourceWithStreamingResponse(self)


class PlatformResourceWithRawResponse:
    def __init__(self, platform: PlatformResource) -> None:
        self._platform = platform

    @cached_property
    def webhooks(self) -> WebhooksResourceWithRawResponse:
        return WebhooksResourceWithRawResponse(self._platform.webhooks)


class AsyncPlatformResourceWithRawResponse:
    def __init__(self, platform: AsyncPlatformResource) -> None:
        self._platform = platform

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithRawResponse:
        return AsyncWebhooksResourceWithRawResponse(self._platform.webhooks)


class PlatformResourceWithStreamingResponse:
    def __init__(self, platform: PlatformResource) -> None:
        self._platform = platform

    @cached_property
    def webhooks(self) -> WebhooksResourceWithStreamingResponse:
        return WebhooksResourceWithStreamingResponse(self._platform.webhooks)


class AsyncPlatformResourceWithStreamingResponse:
    def __init__(self, platform: AsyncPlatformResource) -> None:
        self._platform = platform

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithStreamingResponse:
        return AsyncWebhooksResourceWithStreamingResponse(self._platform.webhooks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from ark import Ark, AsyncArk
from tests.utils import assert_matches_type
from ark.pagination import SyncPageNumberPagination, AsyncPageNumberPagination
from ark.types.tenants import (
    CredentialListResponse,
    CredentialCreateResponse,
    CredentialDeleteResponse,
    CredentialUpdateResponse,
    CredentialRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCredentials:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Ark) -> None:
        credential = client.tenants.credentials.create(
            tenant_id="cm6abc123def456",
            name="production-smtp",
            type="smtp",
        )
        assert_matches_type(CredentialCreateResponse, credential, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Ark) -> None:
        response = client.tenants.credentials.with_raw_response.create(
            tenant_id="cm6abc123def456",
            name="production-smtp",
            type="smtp",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = response.parse()
        assert_matches_type(CredentialCreateResponse, credential, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Ark) -> None:
        with client.tenants.credentials.with_streaming_response.create(
            tenant_id="cm6abc123def456",
            name="production-smtp",
            type="smtp",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = response.parse()
            assert_matches_type(CredentialCreateResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.credentials.with_raw_response.create(
                tenant_id="",
                name="production-smtp",
                type="smtp",
            )

    @parametrize
    def test_method_retrieve(self, client: Ark) -> None:
        credential = client.tenants.credentials.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: Ark) -> None:
        credential = client.tenants.credentials.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
            reveal=True,
        )
        assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Ark) -> None:
        response = client.tenants.credentials.with_raw_response.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = response.parse()
        assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Ark) -> None:
        with client.tenants.credentials.with_streaming_response.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = response.parse()
            assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.credentials.with_raw_response.retrieve(
                credential_id=123,
                tenant_id="",
            )

    @parametrize
    def test_method_update(self, client: Ark) -> None:
        credential = client.tenants.credentials.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Ark) -> None:
        credential = client.tenants.credentials.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
            hold=True,
            name="production-smtp-v2",
        )
        assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Ark) -> None:
        response = client.tenants.credentials.with_raw_response.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = response.parse()
        assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Ark) -> None:
        with client.tenants.credentials.with_streaming_response.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = response.parse()
            assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.credentials.with_raw_response.update(
                credential_id=123,
                tenant_id="",
            )

    @parametrize
    def test_method_list(self, client: Ark) -> None:
        credential = client.tenants.credentials.list(
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(SyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Ark) -> None:
        credential = client.tenants.credentials.list(
            tenant_id="cm6abc123def456",
            page=1,
            per_page=1,
            type="smtp",
        )
        assert_matches_type(SyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Ark) -> None:
        response = client.tenants.credentials.with_raw_response.list(
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = response.parse()
        assert_matches_type(SyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Ark) -> None:
        with client.tenants.credentials.with_streaming_response.list(
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = response.parse()
            assert_matches_type(SyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.credentials.with_raw_response.list(
                tenant_id="",
            )

    @parametrize
    def test_method_delete(self, client: Ark) -> None:
        credential = client.tenants.credentials.delete(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(CredentialDeleteResponse, credential, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Ark) -> None:
        response = client.tenants.credentials.with_raw_response.delete(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = response.parse()
        assert_matches_type(CredentialDeleteResponse, credential, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Ark) -> None:
        with client.tenants.credentials.with_streaming_response.delete(
            credential_id=123,
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = response.parse()
            assert_matches_type(CredentialDeleteResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.credentials.with_raw_response.delete(
                credential_id=123,
                tenant_id="",
            )


class TestAsyncCredentials:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.create(
            tenant_id="cm6abc123def456",
            name="production-smtp",
            type="smtp",
        )
        assert_matches_type(CredentialCreateResponse, credential, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncArk) -> None:
        response = await async_client.tenants.credentials.with_raw_response.create(
            tenant_id="cm6abc123def456",
            name="production-smtp",
            type="smtp",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = await response.parse()
        assert_matches_type(CredentialCreateResponse, credential, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncArk) -> None:
        async with async_client.tenants.credentials.with_streaming_response.create(
            tenant_id="cm6abc123def456",
            name="production-smtp",
            type="smtp",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = await response.parse()
            assert_matches_type(CredentialCreateResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.credentials.with_raw_response.create(
                tenant_id="",
                name="production-smtp",
                type="smtp",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
            reveal=True,
        )
        assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncArk) -> None:
        response = await async_client.tenants.credentials.with_raw_response.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = await response.parse()
        assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncArk) -> None:
        async with async_client.tenants.credentials.with_streaming_response.retrieve(
            credential_id=123,
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = await response.parse()
            assert_matches_type(CredentialRetrieveResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.credentials.with_raw_response.retrieve(
                credential_id=123,
                tenant_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
            hold=True,
            name="production-smtp-v2",
        )
        assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncArk) -> None:
        response = await async_client.tenants.credentials.with_raw_response.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = await response.parse()
        assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncArk) -> None:
        async with async_client.tenants.credentials.with_streaming_response.update(
            credential_id=123,
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = await response.parse()
            assert_matches_type(CredentialUpdateResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.credentials.with_raw_response.update(
                credential_id=123,
                tenant_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.list(
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(AsyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.list(
            tenant_id="cm6abc123def456",
            page=1,
            per_page=1,
            type="smtp",
        )
        assert_matches_type(AsyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArk) -> None:
        response = await async_client.tenants.credentials.with_raw_response.list(
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = await response.parse()
        assert_matches_type(AsyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArk) -> None:
        async with async_client.tenants.credentials.with_streaming_response.list(
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = await response.parse()
            assert_matches_type(AsyncPageNumberPagination[CredentialListResponse], credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.credentials.with_raw_response.list(
                tenant_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncArk) -> None:
        credential = await async_client.tenants.credentials.delete(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )
        assert_matches_type(CredentialDeleteResponse, credential, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncArk) -> None:
        response = await async_client.tenants.credentials.with_raw_response.delete(
            credential_id=123,
            tenant_id="cm6abc123def456",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential = await response.parse()
        assert_matches_type(CredentialDeleteResponse, credential, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncArk) -> None:
        async with async_client.tenants.credentials.with_streaming_response.delete(
            credential_id=123,
            tenant_id="cm6abc123def456",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential = await response.parse()
            assert_matches_type(CredentialDeleteResponse, credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.credentials.with_raw_response.delete(
                credential_id=123,
                tenant_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .email_rates import EmailRates
from .email_counts import EmailCounts

__all__ = ["TenantUsageItem"]


class TenantUsageItem(BaseModel):
    """Usage record for a single tenant (camelCase for SDK)"""

    emails: EmailCounts
    """Email delivery counts"""

    rates: EmailRates
    """Email delivery rates (as decimals, e.g., 0.95 = 95%)"""

    status: Literal["active", "suspended", "archived"]
    """Current tenant status"""

    tenant_id: str = FieldInfo(alias="tenantId")
    """Unique tenant identifier"""

    tenant_name: str = FieldInfo(alias="tenantName")
    """Tenant display name"""

    external_id: Optional[str] = FieldInfo(alias="externalId", default=None)
    """Your external ID for this tenant"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CredentialCreateParams"]


class CredentialCreateParams(TypedDict, total=False):
    name: Required[str]
    """Name for the credential.

    Can only contain letters, numbers, hyphens, and underscores. Max 50 characters.
    """

    type: Required[Literal["smtp", "api"]]
    """Type of credential:

    - `smtp` - For SMTP-based email sending
    - `api` - For API-based email sending
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["UsagePeriod"]


class UsagePeriod(BaseModel):
    """Time period for usage data"""

    end: datetime
    """Period end (inclusive)"""

    start: datetime
    """Period start (inclusive)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.api_meta import APIMeta
from .tenant_usage_timeseries import TenantUsageTimeseries

__all__ = ["UsageRetrieveTimeseriesResponse"]


class UsageRetrieveTimeseriesResponse(BaseModel):
    """Timeseries usage data for a tenant"""

    data: TenantUsageTimeseries
    """Timeseries usage statistics"""

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.api_meta import APIMeta

__all__ = ["DomainListResponse", "Data", "DataDomain"]


class DataDomain(BaseModel):
    id: int
    """Unique domain identifier"""

    name: str
    """The domain name used for sending emails"""

    verified: bool
    """Whether all DNS records (SPF, DKIM, Return Path) are correctly configured.

    Domain must be verified before sending emails.
    """

    tenant_id: Optional[str] = None
    """ID of the tenant this domain belongs to (included when filtering by tenant_id)"""

    tenant_name: Optional[str] = None
    """
    Name of the tenant this domain belongs to (included when filtering by tenant_id)
    """


class Data(BaseModel):
    domains: List[DataDomain]


class DomainListResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageNumberPagination, AsyncPageNumberPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.tenants import (
    credential_list_params,
    credential_create_params,
    credential_update_params,
    credential_retrieve_params,
)
from ...types.tenants.credential_list_response import CredentialListResponse
from ...types.tenants.credential_create_response import CredentialCreateResponse
from ...types.tenants.credential_delete_response import CredentialDeleteResponse
from ...types.tenants.credential_update_response import CredentialUpdateResponse
from ...types.tenants.credential_retrieve_response import CredentialRetrieveResponse

__all__ = ["CredentialsResource", "AsyncCredentialsResource"]


class CredentialsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return CredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return CredentialsResourceWithStreamingResponse(self)

    def create(
        self,
        tenant_id: str,
        *,
        name: str,
        type: Literal["smtp", "api"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialCreateResponse:
        """Create a new SMTP or API credential for a tenant.

        The credential can be used to
        send emails via Ark on behalf of the tenant.

        **Important:** The credential key is only returned once at creation time. Store
        it securely - you cannot retrieve it again.

        **Credential Types:**

        - `smtp` - For SMTP-based email sending. Returns both `key` and `smtpUsername`.
        - `api` - For API-based email sending. Returns only `key`.

        Args:
          name: Name for the credential. Can only contain letters, numbers, hyphens, and
              underscores. Max 50 characters.

          type:
              Type of credential:

              - `smtp` - For SMTP-based email sending
              - `api` - For API-based email sending

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._post(
            f"/tenants/{tenant_id}/credentials",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                },
                credential_create_params.CredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialCreateResponse,
        )

    def retrieve(
        self,
        credential_id: int,
        *,
        tenant_id: str,
        reveal: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialRetrieveResponse:
        """
        Get details of a specific credential.

        **Revealing the key:** By default, the credential key is not returned. Pass
        `reveal=true` to include the key in the response. Use this sparingly and only
        when you need to retrieve the key (e.g., for configuration).

        Args:
          reveal: Set to `true` to include the credential key in the response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get(
            f"/tenants/{tenant_id}/credentials/{credential_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"reveal": reveal}, credential_retrieve_params.CredentialRetrieveParams),
            ),
            cast_to=CredentialRetrieveResponse,
        )

    def update(
        self,
        credential_id: int,
        *,
        tenant_id: str,
        hold: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialUpdateResponse:
        """
        Update a credential's name or hold status.

        **Hold Status:**

        - When `hold: true`, the credential is disabled and cannot be used to send
          emails.
        - When `hold: false`, the credential is active and can send emails.
        - Use this to temporarily disable a credential without deleting it.

        Args:
          hold: Set to `true` to disable the credential (put on hold). Set to `false` to enable
              the credential (release from hold).

          name: New name for the credential

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._patch(
            f"/tenants/{tenant_id}/credentials/{credential_id}",
            body=maybe_transform(
                {
                    "hold": hold,
                    "name": name,
                },
                credential_update_params.CredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialUpdateResponse,
        )

    def list(
        self,
        tenant_id: str,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        type: Literal["smtp", "api"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPageNumberPagination[CredentialListResponse]:
        """List all SMTP and API credentials for a tenant.

        Credentials are used to send
        emails via Ark on behalf of the tenant.

        **Security:** Credential keys are not returned in the list response. Use the
        retrieve endpoint with `reveal=true` to get the key.

        Args:
          page: Page number (1-indexed)

          per_page: Number of items per page (max 100)

          type: Filter by credential type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get_api_list(
            f"/tenants/{tenant_id}/credentials",
            page=SyncPageNumberPagination[CredentialListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "type": type,
                    },
                    credential_list_params.CredentialListParams,
                ),
            ),
            model=CredentialListResponse,
        )

    def delete(
        self,
        credential_id: int,
        *,
        tenant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialDeleteResponse:
        """Permanently delete (revoke) a credential.

        The credential can no longer be used
        to send emails.

        **Warning:** This action is irreversible. If you want to temporarily disable a
        credential, use the update endpoint to set `hold: true` instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._delete(
            f"/tenants/{tenant_id}/credentials/{credential_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialDeleteResponse,
        )


class AsyncCredentialsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return AsyncCredentialsResourceWithStreamingResponse(self)

    async def create(
        self,
        tenant_id: str,
        *,
        name: str,
        type: Literal["smtp", "api"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialCreateResponse:
        """Create a new SMTP or API credential for a tenant.

        The credential can be used to
        send emails via Ark on behalf of the tenant.

        **Important:** The credential key is only returned once at creation time. Store
        it securely - you cannot retrieve it again.

        **Credential Types:**

        - `smtp` - For SMTP-based email sending. Returns both `key` and `smtpUsername`.
        - `api` - For API-based email sending. Returns only `key`.

        Args:
          name: Name for the credential. Can only contain letters, numbers, hyphens, and
              underscores. Max 50 characters.

          type:
              Type of credential:

              - `smtp` - For SMTP-based email sending
              - `api` - For API-based email sending

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._post(
            f"/tenants/{tenant_id}/credentials",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                },
                credential_create_params.CredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialCreateResponse,
        )

    async def retrieve(
        self,
        credential_id: int,
        *,
        tenant_id: str,
        reveal: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialRetrieveResponse:
        """
        Get details of a specific credential.

        **Revealing the key:** By default, the credential key is not returned. Pass
        `reveal=true` to include the key in the response. Use this sparingly and only
        when you need to retrieve the key (e.g., for configuration).

        Args:
          reveal: Set to `true` to include the credential key in the response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._get(
            f"/tenants/{tenant_id}/credentials/{credential_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"reveal": reveal}, credential_retrieve_params.CredentialRetrieveParams
                ),
            ),
            cast_to=CredentialRetrieveResponse,
        )

    async def update(
        self,
        credential_id: int,
        *,
        tenant_id: str,
        hold: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialUpdateResponse:
        """
        Update a credential's name or hold status.

        **Hold Status:**

        - When `hold: true`, the credential is disabled and cannot be used to send
          emails.
        - When `hold: false`, the credential is active and can send emails.
        - Use this to temporarily disable a credential without deleting it.

        Args:
          hold: Set to `true` to disable the credential (put on hold). Set to `false` to enable
              the credential (release from hold).

          name: New name for the credential

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._patch(
            f"/tenants/{tenant_id}/credentials/{credential_id}",
            body=await async_maybe_transform(
                {
                    "hold": hold,
                    "name": name,
                },
                credential_update_params.CredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialUpdateResponse,
        )

    def list(
        self,
        tenant_id: str,
        *,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        type: Literal["smtp", "api"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CredentialListResponse, AsyncPageNumberPagination[CredentialListResponse]]:
        """List all SMTP and API credentials for a tenant.

        Credentials are used to send
        emails via Ark on behalf of the tenant.

        **Security:** Credential keys are not returned in the list response. Use the
        retrieve endpoint with `reveal=true` to get the key.

        Args:
          page: Page number (1-indexed)

          per_page: Number of items per page (max 100)

          type: Filter by credential type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get_api_list(
            f"/tenants/{tenant_id}/credentials",
            page=AsyncPageNumberPagination[CredentialListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "type": type,
                    },
                    credential_list_params.CredentialListParams,
                ),
            ),
            model=CredentialListResponse,
        )

    async def delete(
        self,
        credential_id: int,
        *,
        tenant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialDeleteResponse:
        """Permanently delete (revoke) a credential.

        The credential can no longer be used
        to send emails.

        **Warning:** This action is irreversible. If you want to temporarily disable a
        credential, use the update endpoint to set `hold: true` instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._delete(
            f"/tenants/{tenant_id}/credentials/{credential_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialDeleteResponse,
        )


class CredentialsResourceWithRawResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.create = to_raw_response_wrapper(
            credentials.create,
        )
        self.retrieve = to_raw_response_wrapper(
            credentials.retrieve,
        )
        self.update = to_raw_response_wrapper(
            credentials.update,
        )
        self.list = to_raw_response_wrapper(
            credentials.list,
        )
        self.delete = to_raw_response_wrapper(
            credentials.delete,
        )


class AsyncCredentialsResourceWithRawResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.create = async_to_raw_response_wrapper(
            credentials.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            credentials.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            credentials.update,
        )
        self.list = async_to_raw_response_wrapper(
            credentials.list,
        )
        self.delete = async_to_raw_response_wrapper(
            credentials.delete,
        )


class CredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.create = to_streamed_response_wrapper(
            credentials.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            credentials.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            credentials.update,
        )
        self.list = to_streamed_response_wrapper(
            credentials.list,
        )
        self.delete = to_streamed_response_wrapper(
            credentials.delete,
        )


class AsyncCredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.create = async_to_streamed_response_wrapper(
            credentials.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            credentials.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            credentials.update,
        )
        self.list = async_to_streamed_response_wrapper(
            credentials.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            credentials.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..email_rates import EmailRates
from ..email_counts import EmailCounts
from ..usage_period import UsagePeriod

__all__ = ["TenantUsage"]


class TenantUsage(BaseModel):
    """Tenant usage statistics"""

    emails: EmailCounts
    """Email delivery counts"""

    period: UsagePeriod
    """Time period for usage data"""

    rates: EmailRates
    """Email delivery rates (as decimals, e.g., 0.95 = 95%)"""

    tenant_id: str
    """Unique tenant identifier"""

    tenant_name: str
    """Tenant display name"""

    external_id: Optional[str] = None
    """Your external ID for this tenant (from metadata)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["WebhookCreateParams"]


class WebhookCreateParams(TypedDict, total=False):
    name: Required[str]
    """Display name for the webhook"""

    url: Required[str]
    """Webhook endpoint URL (must be HTTPS)"""

    events: List[
        Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ]
    ]
    """Events to subscribe to. Empty array means all events."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.tenants import usage_retrieve_params, usage_retrieve_timeseries_params
from ...types.tenants.usage_retrieve_response import UsageRetrieveResponse
from ...types.tenants.usage_retrieve_timeseries_response import UsageRetrieveTimeseriesResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def retrieve(
        self,
        tenant_id: str,
        *,
        period: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageRetrieveResponse:
        """
        Returns email sending statistics for a specific tenant over a time period.

        **Use cases:**

        - Display usage dashboard to your customers
        - Calculate per-tenant billing
        - Monitor tenant health and delivery rates

        **Period formats:**

        - Shortcuts: `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
          `last_month`, `last_7_days`, `last_30_days`, `last_90_days`
        - Month: `2024-01` (full month)
        - Date range: `2024-01-01..2024-01-31`
        - Single day: `2024-01-15`

        **Response includes:**

        - `emails` - Counts for sent, delivered, soft_failed, hard_failed, bounced, held
        - `rates` - Delivery rate and bounce rate as decimals (0.95 = 95%)

        Args:
          period: Time period for usage data. Defaults to current month.

              **Formats:**

              - Shortcuts: `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
                `last_month`, `last_7_days`, `last_30_days`, `last_90_days`
              - Month: `2024-01`
              - Range: `2024-01-01..2024-01-31`
              - Day: `2024-01-15`

          timezone: Timezone for period calculations (IANA format). Defaults to UTC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get(
            f"/tenants/{tenant_id}/usage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "period": period,
                        "timezone": timezone,
                    },
                    usage_retrieve_params.UsageRetrieveParams,
                ),
            ),
            cast_to=UsageRetrieveResponse,
        )

    def retrieve_timeseries(
        self,
        tenant_id: str,
        *,
        granularity: Literal["hour", "day", "week", "month"] | Omit = omit,
        period: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageRetrieveTimeseriesResponse:
        """
        Returns time-bucketed email statistics for a specific tenant.

        **Use cases:**

        - Build usage charts and graphs
        - Identify sending patterns
        - Detect anomalies in delivery rates

        **Granularity options:**

        - `hour` - Hourly buckets (best for last 7 days)
        - `day` - Daily buckets (best for last 30-90 days)
        - `week` - Weekly buckets (best for last 6 months)
        - `month` - Monthly buckets (best for year-over-year)

        The response includes a data point for each time bucket with all email metrics.

        Args:
          granularity: Time bucket size for data points

          period: Time period for timeseries data. Defaults to current month.

          timezone: Timezone for period calculations (IANA format). Defaults to UTC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get(
            f"/tenants/{tenant_id}/usage/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "granularity": granularity,
                        "period": period,
                        "timezone": timezone,
                    },
                    usage_retrieve_timeseries_params.UsageRetrieveTimeseriesParams,
                ),
            ),
            cast_to=UsageRetrieveTimeseriesResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        tenant_id: str,
        *,
        period: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageRetrieveResponse:
        """
        Returns email sending statistics for a specific tenant over a time period.

        **Use cases:**

        - Display usage dashboard to your customers
        - Calculate per-tenant billing
        - Monitor tenant health and delivery rates

        **Period formats:**

        - Shortcuts: `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
          `last_month`, `last_7_days`, `last_30_days`, `last_90_days`
        - Month: `2024-01` (full month)
        - Date range: `2024-01-01..2024-01-31`
        - Single day: `2024-01-15`

        **Response includes:**

        - `emails` - Counts for sent, delivered, soft_failed, hard_failed, bounced, held
        - `rates` - Delivery rate and bounce rate as decimals (0.95 = 95%)

        Args:
          period: Time period for usage data. Defaults to current month.

              **Formats:**

              - Shortcuts: `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
                `last_month`, `last_7_days`, `last_30_days`, `last_90_days`
              - Month: `2024-01`
              - Range: `2024-01-01..2024-01-31`
              - Day: `2024-01-15`

          timezone: Timezone for period calculations (IANA format). Defaults to UTC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._get(
            f"/tenants/{tenant_id}/usage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "period": period,
                        "timezone": timezone,
                    },
                    usage_retrieve_params.UsageRetrieveParams,
                ),
            ),
            cast_to=UsageRetrieveResponse,
        )

    async def retrieve_timeseries(
        self,
        tenant_id: str,
        *,
        granularity: Literal["hour", "day", "week", "month"] | Omit = omit,
        period: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageRetrieveTimeseriesResponse:
        """
        Returns time-bucketed email statistics for a specific tenant.

        **Use cases:**

        - Build usage charts and graphs
        - Identify sending patterns
        - Detect anomalies in delivery rates

        **Granularity options:**

        - `hour` - Hourly buckets (best for last 7 days)
        - `day` - Daily buckets (best for last 30-90 days)
        - `week` - Weekly buckets (best for last 6 months)
        - `month` - Monthly buckets (best for year-over-year)

        The response includes a data point for each time bucket with all email metrics.

        Args:
          granularity: Time bucket size for data points

          period: Time period for timeseries data. Defaults to current month.

          timezone: Timezone for period calculations (IANA format). Defaults to UTC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._get(
            f"/tenants/{tenant_id}/usage/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "granularity": granularity,
                        "period": period,
                        "timezone": timezone,
                    },
                    usage_retrieve_timeseries_params.UsageRetrieveTimeseriesParams,
                ),
            ),
            cast_to=UsageRetrieveTimeseriesResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.retrieve = to_raw_response_wrapper(
            usage.retrieve,
        )
        self.retrieve_timeseries = to_raw_response_wrapper(
            usage.retrieve_timeseries,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.retrieve = async_to_raw_response_wrapper(
            usage.retrieve,
        )
        self.retrieve_timeseries = async_to_raw_response_wrapper(
            usage.retrieve_timeseries,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.retrieve = to_streamed_response_wrapper(
            usage.retrieve,
        )
        self.retrieve_timeseries = to_streamed_response_wrapper(
            usage.retrieve_timeseries,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.retrieve = async_to_streamed_response_wrapper(
            usage.retrieve,
        )
        self.retrieve_timeseries = async_to_streamed_response_wrapper(
            usage.retrieve_timeseries,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..usage_period import UsagePeriod

__all__ = ["TenantUsageTimeseries", "Data"]


class Data(BaseModel):
    """Single timeseries data point"""

    bounced: int
    """Bounces in this bucket"""

    delivered: int
    """Emails delivered in this bucket"""

    hard_failed: int
    """Hard failures in this bucket"""

    held: int
    """Emails held in this bucket"""

    sent: int
    """Emails sent in this bucket"""

    soft_failed: int
    """Soft failures in this bucket"""

    timestamp: datetime
    """Start of time bucket"""


class TenantUsageTimeseries(BaseModel):
    """Timeseries usage statistics"""

    data: List[Data]
    """Array of time-bucketed data points"""

    granularity: Literal["hour", "day", "week", "month"]
    """Time bucket granularity"""

    period: UsagePeriod
    """Time period for usage data"""

    tenant_id: str
    """Unique tenant identifier"""

    tenant_name: str
    """Tenant display name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..shared.api_meta import APIMeta

__all__ = ["WebhookTestResponse", "Data"]


class Data(BaseModel):
    duration_ms: int = FieldInfo(alias="durationMs")
    """Request duration in milliseconds"""

    status_code: int = FieldInfo(alias="statusCode")
    """HTTP status code from the webhook endpoint"""

    success: bool
    """Whether the webhook endpoint responded with a 2xx status"""

    error: Optional[str] = None
    """Error message if the request failed"""


class WebhookTestResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CredentialUpdateParams"]


class CredentialUpdateParams(TypedDict, total=False):
    tenant_id: Required[Annotated[str, PropertyInfo(alias="tenantId")]]

    hold: bool
    """
    Set to `true` to disable the credential (put on hold). Set to `false` to enable
    the credential (release from hold).
    """

    name: str
    """New name for the credential"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["UsageRetrieveParams"]


class UsageRetrieveParams(TypedDict, total=False):
    period: str
    """Time period for usage data. Defaults to current month.

    **Formats:**

    - Shortcuts: `today`, `yesterday`, `this_week`, `last_week`, `this_month`,
      `last_month`, `last_7_days`, `last_30_days`, `last_90_days`
    - Month: `2024-01`
    - Range: `2024-01-01..2024-01-31`
    - Day: `2024-01-15`
    """

    timezone: str
    """Timezone for period calculations (IANA format). Defaults to UTC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .emails import (
    EmailsResource,
    AsyncEmailsResource,
    EmailsResourceWithRawResponse,
    AsyncEmailsResourceWithRawResponse,
    EmailsResourceWithStreamingResponse,
    AsyncEmailsResourceWithStreamingResponse,
)
from .limits import (
    LimitsResource,
    AsyncLimitsResource,
    LimitsResourceWithRawResponse,
    AsyncLimitsResourceWithRawResponse,
    LimitsResourceWithStreamingResponse,
    AsyncLimitsResourceWithStreamingResponse,
)
from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from .platform import (
    PlatformResource,
    AsyncPlatformResource,
    PlatformResourceWithRawResponse,
    AsyncPlatformResourceWithRawResponse,
    PlatformResourceWithStreamingResponse,
    AsyncPlatformResourceWithStreamingResponse,
)

__all__ = [
    "EmailsResource",
    "AsyncEmailsResource",
    "EmailsResourceWithRawResponse",
    "AsyncEmailsResourceWithRawResponse",
    "EmailsResourceWithStreamingResponse",
    "AsyncEmailsResourceWithStreamingResponse",
    "LogsResource",
    "AsyncLogsResource",
    "LogsResourceWithRawResponse",
    "AsyncLogsResourceWithRawResponse",
    "LogsResourceWithStreamingResponse",
    "AsyncLogsResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
    "LimitsResource",
    "AsyncLimitsResource",
    "LimitsResourceWithRawResponse",
    "AsyncLimitsResourceWithRawResponse",
    "LimitsResourceWithStreamingResponse",
    "AsyncLimitsResourceWithStreamingResponse",
    "TenantsResource",
    "AsyncTenantsResource",
    "TenantsResourceWithRawResponse",
    "AsyncTenantsResourceWithRawResponse",
    "TenantsResourceWithStreamingResponse",
    "AsyncTenantsResourceWithStreamingResponse",
    "PlatformResource",
    "AsyncPlatformResource",
    "PlatformResourceWithRawResponse",
    "AsyncPlatformResourceWithRawResponse",
    "PlatformResourceWithStreamingResponse",
    "AsyncPlatformResourceWithStreamingResponse",
]
